@@ -19,6 +19,7 @@ See the file COPYING for details.
 #include <string.h>
 #include <errno.h>
 #include <ctype.h>
+#include <unistd.h>
 
 #include <sys/stat.h>
 
@@ -28,6 +29,8 @@ static char * cluster_remove_cmd = NULL;
 static char * cluster_options = NULL;
 static char * cluster_jobname_var = NULL;
 
+int batch_job_verbose_jobnames = 0;
+
 /*
 Principle of operation:
 Each batch job that we submit uses a wrapper file.
@@ -64,7 +67,8 @@ static int setup_batch_wrapper(struct batch_queue *q, const char *sysname )
 	}
 	fchmod(fileno(file), 0755);
 
-	char *path = getenv("PWD");
+	char path[PATH_MAX];
+	getcwd(path, PATH_MAX);
 
 	fprintf(file, "#!/bin/sh\n");
 	fprintf(file, "#$ -S /bin/sh\n");
@@ -104,6 +108,7 @@ static char *cluster_set_resource_string(struct batch_queue *q, const struct rms
 {
 	char *cluster_resources = NULL;
 	const char *ignore_mem = hash_table_lookup(q->options, "ignore-mem-spec");
+	const char *mem_type = hash_table_lookup(q->options, "mem-type");
 
 	if(q->type == BATCH_QUEUE_TYPE_TORQUE || q->type == BATCH_QUEUE_TYPE_PBS){
 		char *mem = string_format(",mem=%" PRId64 "mb", resources->memory);
@@ -125,12 +130,24 @@ static char *cluster_set_resource_string(struct batch_queue *q, const struct rms
 			resources->cores>0 ? resources->cores : 1,
 			(resources->memory>0 && mem) ? mem : "");
 		free(mem);
+	} else if(q->type == BATCH_QUEUE_TYPE_SGE){
+		char *mem = NULL;
+		mem = string_format(" -l %s=%" PRId64 "M", (mem_type ? mem_type : "h_vmem"), resources->memory);
+		// SGE assumes shared FS, ignoring possible disk specification
+		cluster_resources = string_format(" -pe smp %" PRId64 "%s ", 
+			resources->cores>0 ? resources->cores : 1,
+			(resources->memory>0 && mem) ? mem : "");
+		free(mem);
+
 	}
 	const char *safe_mode = hash_table_lookup(q->options, "safe-submit-mode");
-	if(safe_mode && !strcmp("yes", safe_mode))
+	if(safe_mode && !strcmp("yes", safe_mode)) {
 		free(cluster_resources);
-	if(!cluster_resources)
+		cluster_resources = NULL;
+	}
+	if(!cluster_resources) {
 		cluster_resources = xxstrdup("");
+	}
 	return cluster_resources;
 }
 
@@ -185,16 +202,33 @@ static batch_job_id_t batch_job_cluster_submit (struct batch_queue * q, const ch
 	TODO change this to the nodeid during a refactor to batch_task
 	*/
 	static uint16_t submit_id = 0;
+	char *jobname;
 
-	char *command = string_format("%s %s %s %s makeflow%" PRIu16 " %s %s.wrapper",
+	if (batch_job_verbose_jobnames) {
+		char *firstword = strdup(cmd);
+
+		char *end = strchr(firstword, ' ');
+		if (end) *end = 0;
+
+		jobname = strdup(string_front(path_basename(firstword), 15));
+		if (jobname[0] != 0 && !isalpha(jobname[0])) jobname[0] = 'X';
+
+		free(firstword);
+	} else {
+		jobname = string_format("makeflow%" PRIu16, submit_id);
+	}
+	submit_id++;
+
+	char *command = string_format("%s %s %s %s %s %s %s.wrapper",
 		cluster_submit_cmd,
 		cluster_resources,
 		cluster_options,
 		cluster_jobname_var,
-		submit_id++,
+		jobname,
 		options ? options : "",
 		cluster_name);
 
+	free(jobname);
 	free(cluster_resources);
 	debug(D_BATCH, "%s", command);
 
