@@ -11,7 +11,7 @@ cr="./root.$PPID"
 
 check_needed()
 {
-	[ -f ../src/python/_CChirp.so ] || return 1
+	[ -f ../src/bindings/python/_CChirp.so ] || return 1
 	python=$(cctools_python -n 3 2.7 2.6)
 	[ -n "$python" ] || return 1
 	${python} -c "import json; import Chirp"
@@ -32,7 +32,7 @@ run()
 	fi
 	hostport=$(cat "$c")
 
-	../src/python/chirp_jobs_python_example.py $hostport ../src/python/my_script.sh
+	../src/bindings/python/chirp_jobs_python_example.py $hostport ../src/bindings/python/my_script.sh
 
 	return 0
 }
