@@ -11,7 +11,7 @@ ticket=my.ticket
 
 check_needed()
 {
-	[ -f ../src/python/_CChirp.so ] || return 1
+	[ -f ../src/bindings/python/_CChirp.so ] || return 1
 	python=$(cctools_python -n 3 2.7 2.6)
 	[ -n "$python" ] || return 1
 	${python} -c "import json; import Chirp"
@@ -33,7 +33,7 @@ run()
 
 	chirp -d all -a unix "$hostport" ticket_create -output "$ticket" -bits 1024 -duration 86400 -subject unix:`whoami` / write
 
-	../src/python/chirp_python_example.py $hostport $ticket
+	../src/bindings/python/chirp_python_example.py $hostport $ticket
 
 	return 0
 }
