@@ -94,6 +94,8 @@ an example.
 
 #define MAX_REMOTE_JOBS_DEFAULT 100
 
+extern int batch_job_verbose_jobnames;
+
 static sig_atomic_t makeflow_abort_flag = 0;
 static int makeflow_failed_flag = 1; // Makeflow fails by default. This is changed at dag start to indicate correct start.
 static int makeflow_submit_timeout = 3600;
@@ -470,12 +472,16 @@ static int makeflow_node_submit_retry( struct batch_queue *queue, struct batch_t
 		if(makeflow_abort_flag) break;
 
 		/* This will eventually be replaced by submit (queue, task )... */
+		char *input_files  = batch_files_to_string(queue, task->input_files);
+		char *output_files = batch_files_to_string(queue, task->output_files);
 		jobid = batch_job_submit(queue,
 								task->command,
-								batch_files_to_string(queue, task->input_files),
-								batch_files_to_string(queue, task->output_files),
+								input_files,
+								output_files,
 								task->envlist,
 								task->resources);
+		free(input_files);
+		free(output_files);
 
 		if(jobid > 0) {
 			printf("submitted job %"PRIbjid"\n", jobid);
@@ -1009,7 +1015,7 @@ static void handle_abort(int sig)
 	int fd = open("/dev/tty", O_WRONLY);
 	if (fd >= 0) {
 		char buf[256];
-		snprintf(buf, sizeof(buf), "received signal %d (%s), cleaning up remote jobs and files...\n",sig,strsignal(sig));
+		string_nformat(buf, sizeof(buf), "received signal %d (%s), cleaning up remote jobs and files...\n",sig,strsignal(sig));
 		write(fd, buf, strlen(buf));
 		close(fd);
 	}
@@ -1025,55 +1031,60 @@ static void show_help_run(const char *cmd)
 	printf("Use: ./makeflow [options] <dagfile>\n");
 	printf("Basic Options:\n");
 	printf(" -c,--clean=<type>              Clean up logfile and all temporary files.\n");
-	printf(" -d,--debug=<subsystem>         Enable debugging for this subsystem\n");
+	printf("								  all: Cleans all files but inputs (defualt).\n");
+	printf("								  outputs: Cleans output files.\n");
+	printf("								  intermediates: Cleans intermediate files.\n");
+	printf("								  cache: Cleans cache directory.\n");
+	printf(" -d,--debug=<subsystem>         Enable debugging for this subsystem.\n");
 	printf(" -o,--debug-file=<file>         Send debugging to this file.\n");
 	printf("    --debug-rotate-max=<bytes>  Rotate debug file once it reaches this size.\n");
 	printf(" -T,--batch-type=<type>         Select batch system: %s\n",batch_queue_type_string());
-	printf("    --argv=<file>               Include command line arguments from a JSON file.\n");
-	printf(" -v,--version                   Show version string\n");
+	printf("    --argv=<file>               Use command line arguments from a JSON file.\n");
+	printf(" -v,--version                   Show version string.\n");
 	printf(" -h,--help                      Show this help screen.\n");
 	        /********************************************************************************/
 	printf("\nWorkflow Handling:\n");
-	printf(" -a,--advertise                 Advertise workflow status to the global catalog.\n");
+	printf(" -a,--advertise                 Advertise workflow status to catalog.\n");
+	printf(" -l,--makeflow-log=<logfile>    Use this file for the makeflow log.\n");
 	printf(" -L,--batch-log=<logfile>       Use this file for the batch system log.\n");
-	printf(" -m,--email=<email>             Send summary of workflow to this email at end\n");
+	printf(" -m,--email=<email>             Send summary of workflow to this email.\n");
 	printf("    --json                      Use JSON format for the workflow specification.\n");
 	printf("    --jx                        Use JX format for the workflow specification.\n");
-	printf("    --jx-args=<file>            Evaluate the JX input with keys and values in file defined as variables.\n");
-	printf("    --jx-context=<file>         Deprecated. Equivalent to --jx-args.\n");
-	printf("    --jx-define=<VAR>=<EXPR>	Set the JX variable VAR to the JX expression EXPR.\n");
+	printf("    --jx-args=<file>            File defining JX variables for JX workflow.\n");
+	printf("    --jx-define=<VAR>=<EXPR>	Set the JX variable VAR to JX expression EXPR.\n");
 	printf("    --log-verbose               Add node id symbol tags in the makeflow log.\n");
 	printf(" -j,--max-local=<#>             Max number of local jobs to run at once.\n");
 	printf(" -J,--max-remote=<#>            Max number of remote jobs to run at once.\n");
-	printf(" -l,--makeflow-log=<logfile>    Use this file for the makeflow log.\n");
 	printf(" -R,--retry                     Retry failed batch jobs up to 5 times.\n");
 	printf(" -r,--retry-count=<n>           Retry failed batch jobs up to n times.\n");
-	printf("    --send-environment          Send all local environment variables in remote execution.\n");
+	printf("    --send-environment          Send local environment variables for execution.\n");
 	printf(" -S,--submission-timeout=<#>    Time to retry failed batch job submission.\n");
 	printf(" -f,--summary-log=<file>        Write summary of workflow to this file at end.\n");
 	        /********************************************************************************/
 	printf("\nData Handling:\n");
 	printf("    --archive                   Read jobs from and write completed jobs into archive.\n");
+	printf("    --archive-dir=<dir>         Archive directory(/tmp/makeflow.archive.USERID).\n");
 	printf("    --archive-read              Read jobs from archive.\n");
 	printf("    --archive-write             Write jobs into archive.\n");
-	printf("    --archive-dir=<dir>         Base archive directory (default: /tmp/makeflow.archive.USERID).\n");
 	printf(" -A,--disable-afs-check         Disable the check for AFS. (experts only.)\n");
 	printf("    --cache=<dir>               Use this dir to cache downloaded mounted files.\n");
 	printf(" -X,--change-directory=<dir>    Change to <dir> before executing the workflow.\n");
-	printf(" -g,--gc=<type>                 Enable garbage collection. (ref_cnt|on_demand|all)\n");
-	printf("    --gc-size=<int>             Set disk size to trigger GC. (on_demand only)\n");
-	printf(" -G,--gc-count=<int>            Set number of files to trigger GC. (ref_cnt only)\n");
-	printf("    --mounts=<mountfile>        Use this file as a mountlist.\n");
+	printf(" -g,--gc=<type>                 Enable garbage collector.(ref_cnt|on_demand|all)\n");
+	printf("    --gc-size=<int>             Set disk size to trigger GC (on_demand only)\n");
+	printf(" -G,--gc-count=<int>            Set number of files to trigger GC.(ref_cnt only)\n");
+	printf("    --mounts=<mountfile>        Use this file as a mountlist\n");
 	printf("    --skip-file-check           Do not check for file existence before running.\n");
-	printf("    --do-not-save-failed-output Disables moving output of failed nodes to directory.\n"); 
+	printf("    --do-not-save-failed-output Disables saving failed nodes to directory.\n"); 
 	printf("    --shared-fs=<dir>           Assume that <dir> is in a shared filesystem.\n");
-	printf("    --storage-limit=<int>       Set storage limit for Makeflow (default is off)\n");
-	printf("    --storage-type=<type>       Type of storage limit(0:MAX,1:MIN,2:OUTPUT,3:OFF\n");
-	printf("    --storage-print=<file>      Print storage limit calculated by Makeflow\n");
+	printf("    --storage-limit=<int>       Set storage limit for Makeflow.(default is off)\n");
+	printf("    --storage-type=<type>       Storage limit type(0:MAX|1:MIN|2:OUTPUT|3:OFF)\n");
+	printf("    --storage-print=<file>      Print storage limit calculated by Makeflow.\n");
+	printf("    --tickets=<file>            Specify Chirp tickets.\n");
 	printf("    --wait-for-files-upto=<n>   Wait up to <n> seconds for files to be created.\n");
 	printf(" -z,--zero-length-error         Consider zero-length files to be erroneous.\n");
 	        /********************************************************************************/
 	printf("\nWork Queue Options:\n");
+    printf("    --allocation                Specify allocation mode (see manual).\n");
 	printf(" -C,--catalog-server=<hst:port> Select alternate catalog server.\n");
 	printf("    --password                  Password file for authenticating workers.\n");
 	printf(" -p,--port=<port>               Port number to use with Work Queue.\n");
@@ -1087,17 +1098,20 @@ static void show_help_run(const char *cmd)
 	printf(" --work-queue-preferred-connection    Preferred connection: by_ip | by_hostname\n");
 	        /********************************************************************************/
 	printf("\nBatch System Options:\n");
-	printf("    --amazon-config             Amazon EC2 config file from makeflow_ec2_setup.\n");
-	printf("    --lambda-config             Amazon Lambda config file from makeflow_lambda_setup.\n");
-	printf("    --amazon-batch-img          Specify amazon ECS Image (for use with -T amazon-batch)\n");
-	printf("    --amazon-batch-config       Amazon config file from makeflow_amazon_batch_setup.\n");
+	printf("    --amazon-config             Amazon EC2 config from makeflow_ec2_setup.\n");
+	printf("    --lambda-config             Lambda config from makeflow_lambda_setup.\n");
+	printf("    --amazon-batch-config       Batch config from makeflow_amazon_batch_setup.\n");
+	printf("    --amazon-batch-img          Specify Amazon ECS Image(Used for amazon-batch)\n");
 	printf(" -B,--batch-options=<options>   Add these options to all batch submit files.\n");
 	printf("    --disable-cache             Disable batch system caching.\n");
 	printf("    --local-cores=#             Max number of local cores to use.\n");
 	printf("    --local-memory=#            Max amount of local memory (MB) to use.\n");
 	printf("    --local-disk=#              Max amount of local disk (MB) to use.\n");
-	printf("    --safe-submit-mode          Excludes resources at submission (SLURM, TORQUE, and PBS).\n");
+	printf("    --safe-submit-mode          Excludes resources at submission.\n");
+	printf("                                  (SLURM, TORQUE, and PBS)\n");
+	printf("    --verbose-jobnames          Set the job name based on the command.\n");
 	printf("    --ignore-memory-spec        Excludes memory at submission (SLURM).\n");
+	printf("    --batch-mem-type            Specify memory resource type (SGE).\n");
 	printf("    --working-dir=<dir|url>     Working directory for the batch system.\n");
 	        /********************************************************************************/
 	printf("\nContainers and Wrappers:\n");
@@ -1128,6 +1142,7 @@ static void show_help_run(const char *cmd)
 
 	printf("\nResource Monitoring Options:\n");
 	printf(" --monitor=<dir>                Enable resource monitor, write logs to <dir>\n");
+	printf(" --monitor-exe=<file>           Specify monitor executable.\n");
 	printf(" --monitor-interval=<#>         Set monitor interval, in seconds. (default: 1s)\n");
 	printf(" --monitor-with-time-series     Enable monitor time series.\n");
 	printf(" --monitor-with-opened-files    Enable monitoring of opened files.\n");
@@ -1178,6 +1193,7 @@ int main(int argc, char *argv[])
 	char *s;
 	int safe_submit = 0;
 	int ignore_mem_spec = 0;
+	char *batch_mem_type = NULL;
 	category_mode_t allocation_mode = CATEGORY_ALLOCATION_MODE_FIXED;
 	char *mesos_master = "127.0.0.1:5050/";
 	char *mesos_path = NULL;
@@ -1242,6 +1258,7 @@ int main(int argc, char *argv[])
 		LONG_OPT_LOCAL_CORES,
 		LONG_OPT_LOCAL_MEMORY,
 		LONG_OPT_LOCAL_DISK,
+		LONG_OPT_BATCH_MEM_TYPE,
 		LONG_OPT_MONITOR,
 		LONG_OPT_MONITOR_EXE,
 		LONG_OPT_MONITOR_INTERVAL,
@@ -1300,6 +1317,7 @@ int main(int argc, char *argv[])
 		LONG_OPT_MESOS_PRELOAD,
 		LONG_OPT_SEND_ENVIRONMENT,
 		LONG_OPT_K8S_IMG,
+		LONG_OPT_VERBOSE_JOBNAMES,
 	};
 
 	static const struct option long_options_run[] = {
@@ -1326,6 +1344,7 @@ int main(int argc, char *argv[])
 		{"gc-count", required_argument, 0, 'G'},
 		{"help", no_argument, 0, 'h'},
 		{"ignore-memory-spec", no_argument, 0, LONG_OPT_IGNORE_MEM},
+		{"batch-mem-type", required_argument, 0, LONG_OPT_BATCH_MEM_TYPE},
 		{"local-cores", required_argument, 0, LONG_OPT_LOCAL_CORES},
 		{"local-memory", required_argument, 0, LONG_OPT_LOCAL_MEMORY},
 		{"local-disk", required_argument, 0, LONG_OPT_LOCAL_DISK},
@@ -1336,6 +1355,7 @@ int main(int argc, char *argv[])
 		{"monitor-exe", required_argument, 0, LONG_OPT_MONITOR_EXE},
 		{"monitor-interval", required_argument, 0, LONG_OPT_MONITOR_INTERVAL},
 		{"monitor-log-name", required_argument, 0, LONG_OPT_MONITOR_LOG_NAME},
+		{"monitor-log-fmt", required_argument, 0, LONG_OPT_MONITOR_LOG_NAME},
 		{"monitor-measure-dir", no_argument, 0, LONG_OPT_MONITOR_MEASURE_DIR},
 		{"monitor-with-opened-files", no_argument, 0, LONG_OPT_MONITOR_OPENED_FILES},
 		{"monitor-with-time-series",  no_argument, 0, LONG_OPT_MONITOR_TIME_SERIES},
@@ -1352,7 +1372,7 @@ int main(int argc, char *argv[])
 		{"sandbox", no_argument, 0, LONG_OPT_SANDBOX},
 		{"send-environment", no_argument, 0, LONG_OPT_SEND_ENVIRONMENT},
 		{"shared-fs", required_argument, 0, LONG_OPT_SHARED_FS},
-		{"show-output", no_argument, 0, 'O'},
+		{"show-output", no_argument, 0, 'O'}, // Deprecated
 		{"storage-type", required_argument, 0, LONG_OPT_STORAGE_TYPE},
 		{"storage-limit", required_argument, 0, LONG_OPT_STORAGE_LIMIT},
 		{"storage-print", required_argument, 0, LONG_OPT_STORAGE_PRINT},
@@ -1372,17 +1392,17 @@ int main(int argc, char *argv[])
 		{"umbrella-mode", required_argument, 0, LONG_OPT_UMBRELLA_MODE},
 		{"umbrella-spec", required_argument, 0, LONG_OPT_UMBRELLA_SPEC},
 		{"work-queue-preferred-connection", required_argument, 0, LONG_OPT_PREFERRED_CONNECTION},
-		{"wq-estimate-capacity", no_argument, 0, 'E'},
+		{"wq-estimate-capacity", no_argument, 0, 'E'}, // Deprecated
 		{"wq-fast-abort", required_argument, 0, 'F'},
 		{"wq-keepalive-interval", required_argument, 0, 'u'},
 		{"wq-keepalive-timeout", required_argument, 0, 't'},
 		{"wq-schedule", required_argument, 0, 'W'},
-		{"wq-wait-queue-size", required_argument, 0, LONG_OPT_WQ_WAIT_FOR_WORKERS},
+		{"wq-wait-queue-size", required_argument, 0, LONG_OPT_WQ_WAIT_FOR_WORKERS}, // Not advertised
 		{"wrapper", required_argument, 0, LONG_OPT_WRAPPER},
 		{"wrapper-input", required_argument, 0, LONG_OPT_WRAPPER_INPUT},
 		{"wrapper-output", required_argument, 0, LONG_OPT_WRAPPER_OUTPUT},
 		{"zero-length-error", no_argument, 0, 'z'},
-		{"change-directory", required_argument, 0, 'X'},
+		{"change-directory", required_argument, 0, 'X'}, // Deprecated Still functional not advertised
 		{"docker", required_argument, 0, LONG_OPT_DOCKER},
 		{"docker-tar", required_argument, 0, LONG_OPT_DOCKER_TAR},
 		{"docker-opt", required_argument, 0, LONG_OPT_DOCKER_OPT},
@@ -1392,7 +1412,7 @@ int main(int argc, char *argv[])
 		{"amazon-batch-config",required_argument,0,LONG_OPT_AMAZON_BATCH_CFG},
 		{"json", no_argument, 0, LONG_OPT_JSON},
 		{"jx", no_argument, 0, LONG_OPT_JX},
-		{"jx-context", required_argument, 0, LONG_OPT_JX_ARGS},
+		{"jx-context", required_argument, 0, LONG_OPT_JX_ARGS}, // Deprecated
 		{"jx-args", required_argument, 0, LONG_OPT_JX_ARGS},
 		{"jx-define", required_argument, 0, LONG_OPT_JX_DEFINE},
 		{"enforcement", no_argument, 0, LONG_OPT_ENFORCEMENT},
@@ -1407,6 +1427,7 @@ int main(int argc, char *argv[])
 		{"mesos-path", required_argument, 0, LONG_OPT_MESOS_PATH},
 		{"mesos-preload", required_argument, 0, LONG_OPT_MESOS_PRELOAD},
 		{"k8s-image", required_argument, 0, LONG_OPT_K8S_IMG},
+		{"verbose-jobnames", no_argument, 0, LONG_OPT_VERBOSE_JOBNAMES},
 		{0, 0, 0, 0}
 	};
 
@@ -1737,6 +1758,7 @@ int main(int argc, char *argv[])
 				} else {
 					fatal("Allocation mode '%s' is not valid. Use one of: throughput waste fixed");
 				}
+				break;
 			case LONG_OPT_JSON:
 				dag_syntax = DAG_SYNTAX_JSON;
 				break;
@@ -1771,6 +1793,7 @@ int main(int argc, char *argv[])
 			case LONG_OPT_UMBRELLA_SPEC:
 				if(!umbrella) umbrella = makeflow_wrapper_umbrella_create();
 				makeflow_wrapper_umbrella_set_spec(umbrella, (const char *)xxstrdup(optarg));
+				break;
 			case LONG_OPT_MESOS_MASTER:
 				mesos_master = xxstrdup(optarg);
 				break;
@@ -1819,6 +1842,10 @@ int main(int argc, char *argv[])
 			case LONG_OPT_IGNORE_MEM:
 				ignore_mem_spec = 1;
 				break;
+			case LONG_OPT_BATCH_MEM_TYPE:
+				free(batch_mem_type);
+				batch_mem_type = xxstrdup(optarg);
+				break;
 			case LONG_OPT_SAFE_SUBMIT:
 				safe_submit = 1;
 				break;
@@ -1868,6 +1895,9 @@ int main(int argc, char *argv[])
 				jx_delete(j);
 				break;
 			}
+			case LONG_OPT_VERBOSE_JOBNAMES:
+				batch_job_verbose_jobnames = 1;
+				break;
 			default:
 				show_help_run(argv[0]);
 				return 1;
@@ -2077,6 +2107,7 @@ int main(int argc, char *argv[])
 	batch_queue_set_option(remote_queue, "amazon-batch-img", amazon_batch_img);
 	batch_queue_set_option(remote_queue, "safe-submit-mode", safe_submit ? "yes" : "no");
 	batch_queue_set_option(remote_queue, "ignore-mem-spec", ignore_mem_spec ? "yes" : "no");
+	batch_queue_set_option(remote_queue, "mem-type", batch_mem_type);
 
 	char *fa_multiplier = string_format("%f", wq_option_fast_abort_multiplier);
 	batch_queue_set_option(remote_queue, "fast-abort", fa_multiplier);
