@@ -46,6 +46,11 @@ extern "C" {
 #include <sys/stat.h>
 #include <sys/time.h>
 
+#ifndef major
+/* glibc 2.26 drops sys/sysmacros.h from sys/types.h, thus we add it here */
+#include <sys/sysmacros.h>
+#endif
+
 #include <assert.h>
 #include <ctype.h>
 #include <errno.h>
@@ -511,6 +516,9 @@ int pfs_table::resolve_name(int is_special_syscall, const char *cname, struct pf
 		char tmp[PFS_PATH_MAX];
 		path_split(pname->path,pname->service_name,tmp);
 		pname->service = pfs_service_lookup(pname->service_name);
+		if (result == PFS_RESOLVE_LOCAL) {
+			pname->service = NULL;
+		}
 		if(!pname->service) {
 			pname->service = pfs_service_lookup_default();
 			strcpy(pname->service_name,"local");
@@ -558,7 +566,9 @@ int pfs_table::resolve_name(int is_special_syscall, const char *cname, struct pf
 
 		if(pattern_match(pname->path, "^/proc/self/?()", &n) >= 0) {
 			strncpy(full_logical_name, pname->path, sizeof(full_logical_name));
-			snprintf(pname->path, sizeof(pname->path), "/proc/%d/%s", pfs_process_getpid(), &full_logical_name[n]);
+
+			string_nformat(pname->path, sizeof(pname->path), "/proc/%d/%s", pfs_process_getpid(), &full_logical_name[n]);
+
 			strcpy(pname->logical_name, pname->path);
 			strcpy(pname->rest, pname->path);
 			pname->service = pfs_service_lookup_default();
@@ -568,7 +578,7 @@ int pfs_table::resolve_name(int is_special_syscall, const char *cname, struct pf
 			pname->is_local = 1;
 		} else if (pattern_match(pname->path, "^/dev/fd/?()", &n) >= 0) {
 			strncpy(full_logical_name, pname->path, sizeof(full_logical_name));
-			snprintf(pname->path, sizeof(pname->path), "/proc/%d/fd/%s", pfs_process_getpid(), &full_logical_name[n]);
+			string_nformat(pname->path, sizeof(pname->path), "/proc/%d/fd/%s", pfs_process_getpid(), &full_logical_name[n]);
 			strcpy(pname->logical_name, pname->path);
 			strcpy(pname->rest, pname->path);
 			pname->service = pfs_service_lookup_default();
