@@ -67,7 +67,8 @@ typedef enum {
 	WORK_QUEUE_RESULT_FORSAKEN            = 5 << 3, /**< The task failed, but it was not a task error **/
 	WORK_QUEUE_RESULT_MAX_RETRIES         = 6 << 3, /**< The task could not be completed successfully in the given number of retries. **/
 	WORK_QUEUE_RESULT_TASK_MAX_RUN_TIME   = 7 << 3, /**< The task ran for more than the specified time (relative since running in a worker). **/
-	WORK_QUEUE_RESULT_DISK_ALLOC_FULL     = 8 << 3  /**< The task filled its loop device allocation but needed more space. **/
+	WORK_QUEUE_RESULT_DISK_ALLOC_FULL     = 8 << 3, /**< The task filled its loop device allocation but needed more space. **/
+	WORK_QUEUE_RESULT_RMONITOR_ERROR      = 9 << 3  /**< The task failed because the monitor did not produce a summary report. **/
 } work_queue_result_t;
 
 typedef enum {
@@ -469,10 +470,9 @@ void work_queue_task_specify_category(struct work_queue_task *t, const char *cat
 /** Label the task with a user-defined feature. The task will only run on a worker that provides (--feature option) such feature.
 @param q A work queue object.
 @param t A task object.
-@param count The number of resources consumed. If 0, the task does not consume the resource, but the worker still needs to provide it.
-@param category The name of the feature.
+@param feature The name of the feature.
 */
-void work_queue_task_specify_feature(struct work_queue_task *t, const char *name, int64_t count);
+void work_queue_task_specify_feature(struct work_queue_task *t, const char *name);
 
 /** Specify the priority of this task relative to others in the queue.
 Tasks with a higher priority value run first. If no priority is given, a task is placed at the end of the ready list, regardless of the priority.
@@ -797,6 +797,16 @@ multiplier.  The value specified here applies only to tasks in the given categor
 */
 int work_queue_activate_fast_abort_category(struct work_queue *q, const char *category, double multiplier);
 
+
+/** Set the draining mode per worker hostname.
+	If drain_flag is 0, workers at hostname receive tasks as usual.
+    If drain_flag is not 1, no new tasks are dispatched to workers at hostname,
+    and if empty they are shutdown.
+  @param q A work queue object.
+  @param drain_flag Draining mode.
+  */
+int work_queue_specify_draining_by_hostname(struct work_queue *q, const char *hostname, int drain_flag);
+
 /** Turn on or off first-allocation labeling for a given category. By default, only cores, memory, and disk are labeled. Turn on/off other specific resources use @ref work_queue_specify_category_autolabel_resource.
 @param q A work queue object.
 @param category A category name.
